**GHOST** (**G**lobally **H**armonised **O**bservations in **S**pace & **T**ime) 
is a project dedicated to the harmonisation of global surface observations (most 
notably air quality pollutants) and metadata, for the purpose of faciliting a greater
quality of observational/model comparison in the atmospheric chemistry community.

Some of the unique features of **GHOST** are:       
*All relevant network provided numerical and textual metadata is standardised.    
*All metadata parameters are handled as dymnaically changing variables though time.  
*Data of all native temporal resolutions are handled and gridded onto consistent hourly, 3hourly, 6hourly, daily and monthly resolutions, as well as hourly instantaneous, 3hourly instantaneous and 6 hourly instantaneous measurements for relevant instantaneous measurements.     
*Reported measurement methodologies are referenced to a centrally compiled measurement methodology database, allowing the standardisation of measurement methodologies.  
*Network provided station classifications are carefully standardised.    
*All data is passed through numerous quality control tests, giving detailed quality assurance (QA) flags.

Progress on the project is documented here: https://docs.google.com/spreadsheets/d/1n9g3cTwfeHRhXJ9GwnjZE6AF_3amb8O_mjhVVVnPrYE/edit?usp=sharing

A summary of the networks currently processed and contained under the umbrella of **GHOST** 
are given here:

| **Network**           | **Update Frequency**  | **Data Source** | **Data Rights** | **Last Data Download** | **Contact Email** | **Other Links** | **Comments** |
| ---                   | ---                  | ---             | ---              | ---                    | ---               | ---             | ---          |
| AERONET v3        | Daily | https://aeronet.gsfc.nasa.gov/new_web/data.html | https://aeronet.gsfc.nasa.gov/new_web/data_usage.html | CES daily download, i.e. previous day | Brent.N.Holben@nasa.gov, David.M.Giles@nasa.gov | Station List: https://aeronet.gsfc.nasa.gov/aeronet_locations_v3.txt<br/>Station Information: https://aeronet.gsfc.nasa.gov/cgi-bin/site_info_v3<br/>Measurement System Description:https://aeronet.gsfc.nasa.gov/new_web/system_descriptions_instrument.html | |
| CANADA NAPS           | Annual | http://data.ec.gc.ca/data/air/monitor/national-air-pollution-surveillance-naps-program/Data-Donnees/?lang=en | https://open.canada.ca/en/open-government-licence-canada | 27/07/2020 | ec.info-donnees-rnspa-naps-data-info.ec@canada.ca | Program website: https://www.canada.ca/en/environment-climate-change/services/air-pollution/monitoring-networks-data/national-air-pollution-program.html | Network has speciated aerosol/VOC measurements but are badly formatted in .xlsv formats and would need significant efforts to process |
| CAPMoN                | Annual | http://data.ec.gc.ca/data/air/monitor/monitoring-of-atmospheric-gases/ground-level-ozone/?lang=en |  https://open.canada.ca/en/open-government-licence-canada  | 27/07/2020 | ec.rcepa-capmon.ec@canada.ca | | |
| CHILE SINCA           | Daily | https://sinca.mma.gob.cl/index.php/ | None Stated | 19/08/2020 | oficinadepartesmma@mma.gob.cl | | |              
| EANET                 | Annual | https://monitoring.eanet.asia/document/menu/index | https://monitoring.eanet.asia/document/public/index | 29/07/2020 | eanetdata@acap.asia | Site information: https://www.eanet.asia/about/site-information/ | The measurement position of stations sometimes move. The traceability of this process is not well done, and sometimes station codes change and sometimes not  |
| EBAS              | Daily | http://ebas.nilu.no/default.aspx<br/>Data extract made by paul.eckhardt@nilu.no | Information under the Data Policy and Framework tabs (have downloaded data for all open frameworks): http://ebas.nilu.no/default.aspx | 11/03/2020 | ebas@nilu.no, paul.eckhardt@nilu.no | Data Format: https://ebas-submit.nilu.no<br/>GAWSIS station information: https://gawsis.meteoswiss.ch/GAWSIS/#/| Data extract was made by Paul Eckhardt to ensure variables are separated by file. |
| EEA AirBase           | Defunct | https://www.eea.europa.eu/data-and-maps/data/airbase-the-european-air-quality-database-8 | https://www.eea.europa.eu/legal/eea-data-policy | | helpdesk@eionet.europa.eu | European Air Quality Portal: https://aqportal.discomap.eea.europa.eu | |
| EEA AQ-eReporting | Daily | https://www.eea.europa.eu/data-and-maps/data/aqereporting-8 | https://www.eea.europa.eu/legal/eea-data-policy | | helpdesk@eionet.europa.eu | E1a/E2a metadata: https://discomap.eea.europa.eu/map/fme/metadata/PanEuropean_metadata.csv<br/>AIDE D metadata: http://aided.apps.eea.europa.eu/?source=%7B%22query%22%3A%7B%22match_all%22%3A%7B%7D%7D%2C%22display_type%22%3A%22tabular%22%7D<br/>European Air Quality Portal: https://aqportal.discomap.eea.europa.eu | | 
| JAPAN NIES            | Annual | https://tenbou.nies.go.jp/download/ | Bottom of page: https://translate.google.com/translate?hl=&sl=ja&tl=en&u=https%3A%2F%2Fwww.nies.go.jp%2Figreen%2F | 07/08/2020 | gis@nies.go.jp | | All Metadata documents translated to English from Japanese | 
| MEXICO CDMX           | annual | http://www.aire.cdmx.gob.mx/default.php?opc=%27aKBhnmI=%27&opcion=Zg== | None Stated | 19/08/2020/ | calidadaire@sedema.cdmx.gob.mx | Landing page: http://www.aire.cdmx.gob.mx/default.php | | 
| NOAA ISD          | daily | https://www.ncei.noaa.gov/data/global-hourly/ | https://catalog.data.gov/dataset/integrated-surface-global-hourly-data<br/>https://www.usa.gov/government-works | 22/07/2020 | ncei.orders@noaa.gov | Data Format: https://www.ncei.noaa.gov/data/global-hourly/doc/isd-format-document.pdf<br/>Station Metadata: ftp://ftp.ncdc.noaa.gov/pub/data/noaa/isd-history.txt | |
| SEARCH                | defunct | https://www.dropbox.com/sh/o9hxoa4wlo97zpe/AACbm6LetQowrpUgX4vUxnoDa?dl=0 | Now Defunct network | 03/08/2020 | eedgerton@atmospheric-research.com<br/>ahansen@epri.com | Network Overview: https://yosemite.epa.gov/sab/sabproduct.nsf/B99E5E7B1FC13632852575B50069AAC2/$File/EPRI-SEARCH+for+EPA+SAB+INC+May+14-15+2009+Meeting.pdf<br/>Network Paper 1 (Station Metadata drawn from here): https://www.tandfonline.com/doi/abs/10.1080/10473289.2003.10466318<br/>Network Paper 2: https://pubmed.ncbi.nlm.nih.gov/16295278/<br/>Network Paper 3: https://www.tandfonline.com/doi/abs/10.1080/10473289.2006.10464585<br/>Trend Paper: https://www.tandfonline.com/doi/full/10.1080/10962247.2012.748523 | Variable data formats per parameter (e.g. .csv/.xls).<br/>Data incorporates measurements from NYSERDA and SOAS.<br/No clear public information on status of network.<br/>The Public page is unresponsive: http://www.atmospheric-research.com/studies/search/FS<br/>Here it is suggested the network has been decommissioned: https://agupubs.onlinelibrary.wiley.com/doi/full/10.1029/2017JD028074<br/>Here is the paper which provides a dropbox link to the network data: https://acp.copernicus.org/articles/18/8183/2018/   
| UK AIR                | daily | https://uk-air.defra.gov.uk/networks/find-sites?view=advanced | Bottom of page: https://uk-air.defra.gov.uk/data/ | 31/07/2020 | |           
| US EPA AQS            | daily | https://aqs.epa.gov/aqsweb/airdata/download_files.html | https://edg.epa.gov/EPA_Data_License.html<br/>https://aqs.epa.gov/aqsweb/documents/data_mart_welcome.html | 27/07/2020 | https://www.epa.gov/aqs/forms/aqs-contact-us | Station Metadata: https://aqs.epa.gov/aqsweb/airdata/aqs_sites.zip<br/>Monitor Metadata: https://aqs.epa.gov/aqsweb/airdata/aqs_monitors.zip<br/>Metadata Codes: https://www.epa.gov/aqs/aqs-code-list |
| US EPA CASTNET        | annual | https://java.epa.gov/castnet/epa_jsp/prepackageddata.jsp | https://edg.epa.gov/EPA_Data_License.html<br/>https://www.epa.gov/castnet/data-use-and-citation | 27/07/2020 | https://www.epa.gov/castnet/forms/contact-us-about-clean-air-status-and-trends-network | Station Metadata: ftp://newftp.epa.gov/Castnet/CASTNET_Outgoing/data/site.zip<br/> Station Metadata 2: https://www.epa.gov/castnet/castnet-site-locations | Stated that aerosol chemical speciation measurements (vischem) stopped in December 2001, and is now continued in the IMPROVE network: http://vista.cira.colostate.edu/improve |     
| US NADP AMNet                | daily | https://nadp.slh.wisc.edu/networks/atmospheric-mercury-network/ | http://nadp.slh.wisc.edu/nadp/useConditions.aspx | | |   
| US NADP AMoN          | daily | https://nadp.slh.wisc.edu/networks/ammonia-monitoring-network/ | http://nadp.slh.wisc.edu/nadp/useConditions.aspx | 31/07/2020 | David.Gay@slh.wisc.edu | | |
| WMO WDCGG             | annual | https://gaw.kishou.go.jp/policy/gaw | | 30/07/2020 | wdcgg@met.kishou.go.jp | Metadata definitions: https://gaw.kishou.go.jp/documents/db_list<br/>GAWSIS station information: https://gawsis.meteoswiss.ch/GAWSIS/#/ | Other reactive gases previously hosted here but now by EBAS, see note: https://www.gaw-wdcrg.org/default.aspx | 
